package core.repositories.timemanager;

import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.time.LocalDate;
import java.util.List;

@Repository
public interface TaskRepository extends CrudRepository<Task, Long> {

    @Query(value = "SELECT t FROM Task t WHERE t.taskgroup.user.username = ?1 AND t.finished = ?2")
    List<Task> findAllByUser(String username, boolean finished);


    @Modifying
    @Transactional
    void deleteAllByTaskgroup(Taskgroup taskgroup);

    @Transactional
    @Modifying
    void deleteByTaskID(long taskID);

    @Query(value = "SELECT t FROM Task t WHERE t.taskgroup.user.username = ?1 AND t.deadline is NOT NULL AND t.deadline < ?2 AND t.finished = FALSE")
    List<Task> findAllOverdue(String username, LocalDate now);

    @Query(value = "SELECT t FROM Task t WHERE t.taskgroup.user.username = ?1 AND t.startDate IS NOT NULL AND t.startDate > ?2 AND t.finished = FALSE")
    List<Task> findAllUpcoming(String username, LocalDate now);

    @Query(value = "SELECT t FROM Task t WHERE t.taskgroup.user.username = ?1 AND (t.startDate IS NULL OR t.startDate <= ?2) AND t.finished = FALSE")
    List<Task> findAllActive(String username, LocalDate now);
}
